﻿using Infragistics.Core.Graphics;
using Infragistics.Scheduler;
using Infragistics.Scheduler.Data;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using App5.Models;
#if AndroidNative
using Scheder = Infragistics.Controls.Scheduler.SchedulerView;
using Infragistics.Controls.Scheduler;
#else
using Infragistics.XamarinForms.Controls.Scheduler;
using Scheder = Infragistics.XamarinForms.Controls.Scheduler.XamScheduler;
#endif

namespace App5.Data
{
    public static class DataSourceManager
    {
        /// <summary>
        /// ScheduleAppointment and ScheduleResource
        /// </summary>
        /// <returns></returns>
        public static ScheduleListDataSource GetSource()
        {
            DateTime date = DateTime.Now.Date;
            ObservableCollection<Appointment> appts = new ObservableCollection<Appointment>();


            // Create some resources
            ObservableCollection<ScheduleResource> resources = new ObservableCollection<ScheduleResource>();
            resources.Add(new ScheduleResource("ID1", "Bob Smith") { ColorScheme = ScheduleResourceColorScheme.Chocolate });
            resources.Add(new ScheduleResource("ID2", "Joe Davis") { ColorScheme = ScheduleResourceColorScheme.Fuchsia });
            resources.Add(new ScheduleResource("ID3", "Tim Jones") { ColorScheme = ScheduleResourceColorScheme.Grape });

            Func<DateTime, int, int, int, DateTime> MakeDateTime = (d, h, m, s) =>
            {
                return new DateTime(d.Year, d.Month, d.Day, h, m, s);
            };

            // Appointments contained in a single day.
            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date, 12, 15, 0),
                Subject = "Daily Meeting 1",
                Location = "Conference Room 1",
                ResourceId = resources[0].Id,
            });

            appts.Add(new Appointment
            {

                Start = MakeDateTime(date, 13, 0, 0),
                End = MakeDateTime(date, 13, 15, 0),
                Subject = "Daily Meeting 2",
                Location = "Conference Room 2",
                ResourceId = resources[1].Id,
            });

            appts.Add(new Appointment
            {
                Start = DateTime.Now.AddMinutes(15),
                End = DateTime.Now.AddMinutes(30),
                Subject = "Daily Meeting 3",
                Location = "Conference Room 3",
                ResourceId = resources[0].Id,

            });

            // Appointments that span 2 days.
            date = date.AddDays(2);    // Start 2 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "New Daily Meeting 1",
                Location = "New Conference Room 1",
                ResourceId = resources[1].Id,

            });

            // Appointments that span 3 days.
            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 1",
                Location = "Updated Conference Room 1",
                ResourceId = resources[2].Id,
            });

            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 2",
                Location = "Updated Conference Room 2",
                ResourceId = resources[1].Id,
            });

            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 3",
                Location = "Updated Conference Room 3",
                ResourceId = resources[0].Id,
            });

            ScheduleListDataSource lds = new ScheduleListDataSource();
            lds.ResourceItemsSource = resources;
            lds.AppointmentItemsSource = appts;

            return lds;
        }

        public static ObservableCollection<Appointment> GetAppointments()
        {

            DateTime date = DateTime.Now.Date;
            ObservableCollection<Appointment> appts = new ObservableCollection<Appointment>();

            Func<DateTime, int, int, int, DateTime> MakeDateTime = (d, h, m, s) =>
            {
                return new DateTime(d.Year, d.Month, d.Day, h, m, s);
            };

            // Appointments contained in a single day.
            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date, 12, 15, 0),
                Subject = "Daily Meeting 1",
                Location = "Conference Room 1",
                ResourceId = "ID3",
                //Resource = resources[0]
            });

            appts.Add(new Appointment
            {

                Start = MakeDateTime(date, 13, 0, 0),
                End = MakeDateTime(date, 13, 15, 0),
                Subject = "Daily Meeting 2",
                Location = "Conference Room 2",
                ResourceId = "ID2",

                //Resource = resources[1]
            });

            appts.Add(new Appointment
            {
                Start = DateTime.Now.AddMinutes(15),
                End = DateTime.Now.AddMinutes(30),
                Subject = "Daily Meeting 3",
                Location = "Conference Room 3",
                ResourceId = "ID12",

            });

            // Appointments that span 2 days.
            date = date.AddDays(2);    // Start 2 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "New Daily Meeting 1",
                Location = "New Conference Room 1",
                ResourceId = "ID2",
                //Resource = resources[2]

            });

            // Appointments that span 3 days.
            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 1",
                Location = "Updated Conference Room 1",
                ResourceId = "ID1",
                //Resource = resources[0]

            });

            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 2",
                Location = "Updated Conference Room 2",
                ResourceId = "ID3",
                //Resource = resources[1]
            });

            date = date.AddDays(5);    // Start 5 days later

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 12, 0, 0),
                End = MakeDateTime(date.AddDays(2), 12, 15, 0),
                Subject = "Updated Daily Meeting 3",
                Location = "Updated Conference Room 3",
                //Resource = resources[0]
            });

            return appts;
        }

        public static ObservableCollection<CustomAppointmentViewModel> GetCustomAppointmentViewModel()
        {
            DateTime date = DateTime.Now.Date;
            ObservableCollection<CustomAppointmentViewModel> appts = new ObservableCollection<CustomAppointmentViewModel>();

            Func<DateTime, int, int, int, DateTime> MakeDateTime = (d, h, m, s) =>
            {
                return new DateTime(d.Year, d.Month, d.Day, h, m, s);
            };

            for (int i = 0; i < 3; i++)
            {
                appts.Add(new CustomAppointmentViewModel
                {
                    CustomId = Guid.NewGuid().ToString(),
                    CustomStart = MakeDateTime(date.AddDays(i), 12, 0, 0),
                    CustomEnd = MakeDateTime(date.AddDays(i), 12, 15, 0),
                    CustomSubject = "GetCustomDataSource-Custom subject" + i,
                    CustomLocation = "GetCustomDataSource-Custom location" + i,
                    CustomResourceId = "ID1",
                    CustomDescription = "GetCustomDataSource-Here is custom description" + i,
                });
            }

            DateRecurrence recurrence = new DateRecurrence(DateRecurrenceFrequency.Daily);
            recurrence.Count = 30;
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule(DayOfWeek.Wednesday));
            recurrence.Interval = 2;

            appts.Add(new CustomAppointmentViewModel
            {
                CustomId = Guid.NewGuid().ToString(),
                CustomStart = MakeDateTime(date, 0, 0, 0),
                CustomEnd = MakeDateTime(date, 0, 15, 0),
                CustomSubject = "Rec-Custom subject",
                CustomLocation = "Rec-Custom location",
                CustomResourceId = "ID2",
                CustomDescription = "Rec-Here is custom description",
                CustomRecurrence = recurrence.ToICalendarString()
            });

            recurrence = new DateRecurrence(DateRecurrenceFrequency.Daily);
            recurrence.Count = 30;
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule(DayOfWeek.Friday));
            recurrence.Interval = 2;

            appts.Add(new CustomAppointmentViewModel
            {
                CustomId = Guid.NewGuid().ToString(),
                CustomStart = MakeDateTime(date, 0, 0, 0),
                CustomEnd = MakeDateTime(date, 0, 15, 0),
                CustomSubject = "Rec-Custom subject",
                CustomLocation = "Rec-Custom location",
                CustomResourceId = "ID1",
                CustomDescription = "Rec-Here is custom description",
                CustomRecurrence = recurrence.ToICalendarString()
            });

            return appts;
        }

        public static ObservableCollection<CustomResourcesViewModel> GetCustomResourcesViewModels()
        {
            ObservableCollection<CustomResourcesViewModel> resources = new ObservableCollection<CustomResourcesViewModel>();
            CustomResourcesViewModel resource1 = (new CustomResourcesViewModel("ID1", "Custom display") { CustomColorScheme = "Indigo" });
            CustomResourcesViewModel resource2 = (new CustomResourcesViewModel("ID2", "Custom display 2") { CustomColorScheme = "Strawberry" });
            CustomResourcesViewModel resource3 = (new CustomResourcesViewModel("ID3", "Custom display 3") { CustomColorScheme = "Teal" });
            resources.Add(resource1);
            resources.Add(resource2);
            resources.Add(resource3);

            return resources;
        }

        /// <summary>
        /// ScheduleAppointment and ScheduleResource
        /// </summary>
        /// <returns></returns>
        public static ScheduleListDataSource GetRecurrentSource()
        {
            DateTime date = DateTime.Now.Date;
            ObservableCollection<Appointment> appts = new ObservableCollection<Appointment>();

            // Create some resources
            ObservableCollection<ScheduleResource> resources = new ObservableCollection<ScheduleResource>();
            resources.Add(new ScheduleResource("ID1", "Bob Smith") { ColorScheme = ScheduleResourceColorScheme.Lime });
            resources.Add(new ScheduleResource("ID2", "Joe Davis") { ColorScheme = ScheduleResourceColorScheme.Chocolate });
            resources.Add(new ScheduleResource("ID3", "Tim Jones") { ColorScheme = ScheduleResourceColorScheme.Strawberry });

            Func<DateTime, int, int, int, DateTime> MakeDateTime = (d, h, m, s) =>
            {
                return new DateTime(d.Year, d.Month, d.Day, h, m, s);
            };

            DateRecurrence recurrence = new DateRecurrence(DateRecurrenceFrequency.Daily);

            //recurrence.Count = 50;
            //recurrence.Interval = 2;
            //recurrence.Until = MakeDateTime(new DateTime(2017, 4, 26), 16, 0, 0);
            //recurrence.WeekStart = DayOfWeek.Sunday;
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule() { Day = DayOfWeek.Monday });
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule() { Day = DayOfWeek.Tuesday });
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule() { Day = DayOfWeek.Wednesday });
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule() { Day = DayOfWeek.Thursday });
            recurrence.Rules.Add(new DayOfWeekRecurrenceRule() { Day = DayOfWeek.Friday });

            var resourceString = recurrence.ToICalendarString();
            // Appointments contained in a single day.
            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 16, 00, 0),
                //End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "Daily Meeting 1",
                Location = "Conference Room 1",
                ResourceId = resources[2].Id,
            });

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date, 16, 00, 0),
                //End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "Daily Meeting 1",
                Location = "Conference Room 1",
                ResourceId = resources[0].Id,
                Recurrence = resourceString,
            });

            recurrence = new DateRecurrence(DateRecurrenceFrequency.Hourly);
            recurrence.Count = 8;
            recurrence.Interval = 2;
            resourceString = recurrence.ToICalendarString();

            appts.Add(new Appointment
            {
                Start = MakeDateTime(date.AddDays(1), 15, 00, 0),
                //End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "Daily Meeting 2",
                Location = "Conference Room 2",
                ResourceId = resources[1].Id,
                Recurrence = resourceString,
            });

            recurrence = new DateRecurrence(DateRecurrenceFrequency.Weekly);
            recurrence.Interval = 2;
            recurrence.WeekStart = DayOfWeek.Wednesday;
            recurrence.Until = DateTime.Now.AddMonths(1);
            resourceString = recurrence.ToICalendarString();
            appts.Add(new Appointment
            {
                Start = MakeDateTime(date.AddDays(1), 15, 00, 0),
                //End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                Subject = "Daily Meeting 3",
                Location = "Conference Room 3",
                ResourceId = resources[2].Id,
                Recurrence = resourceString,
            });

            ScheduleListDataSource lds = new ScheduleListDataSource();
            lds.ResourceItemsSource = resources;
            lds.AppointmentItemsSource = appts;

            return lds;
        }

        public static void AddAppDynamically(Scheder scheduler)
        {
            DateTime date = DateTime.Now.Date;
            ObservableCollection<Appointment> appts = new ObservableCollection<Appointment>();

            if (scheduler.DataSource == null)
            {
                scheduler.DataSource = new ScheduleListDataSource();
                (scheduler.DataSource as ScheduleListDataSource).AppointmentItemsSource = new ObservableCollection<Appointment>();
                (scheduler.DataSource as ScheduleListDataSource).ResourceItemsSource = new ObservableCollection<ScheduleResource>();
            }
            if (((scheduler.DataSource as ScheduleListDataSource).ResourceItemsSource as ObservableCollection<ScheduleResource>).Count == 0)
            {
                // Create some resources
                ((scheduler.DataSource as ScheduleListDataSource)
                    .ResourceItemsSource as ObservableCollection<ScheduleResource>).Add(new ScheduleResource("ID1",
                    "Bob Smith")
                    { ColorScheme = ScheduleResourceColorScheme.Lime });
            }

            Func<DateTime, int, int, int, DateTime> MakeDateTime = (d, h, m, s) =>
            {
                return new DateTime(d.Year, d.Month, d.Day, h, m, s);
            };

            //Create Recurrence 
            DateRecurrence recurrence = new DateRecurrence(DateRecurrenceFrequency.Minutely);
            recurrence.Count = 7;
            recurrence.Interval = 3;

            var resourceString = recurrence.ToICalendarString();
            // Appointments contained in a single day.

            ScheduleListDataSource listDataSource = new ScheduleListDataSource();
            ObservableCollection<Appointment> appointments = new ObservableCollection<Appointment>();
            appointments.Add(new Appointment()
            {
                Start = DateTime.Now,
                End = DateTime.Now,
                Subject = "Daily Meeting 1",
                Location = "Conference Room 1",
                ResourceId = "ID1",
                Recurrence = resourceString,
            });

            scheduler.DataSource = listDataSource;

            ((scheduler.DataSource as ScheduleListDataSource)
                .AppointmentItemsSource as ObservableCollection<Appointment>).Add(new Appointment
                {
                    Start = MakeDateTime(date, 12, 0, 0),
                    End = MakeDateTime(date.AddDays(1), 12, 15, 0),
                    Subject = "Daily Meeting 1",
                    Location = "Conference Room 1",
                    ResourceId = scheduler.DataSource.GetResources().ElementAt(0).Id,
                    Recurrence = resourceString,
                });
        }
    }

}
